# coding=utf-8
#
# Copyright (C) 2011 Karlisson Bezerra <contact@hacktoon.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Convas module for ink2canvas extension
"""

from inkex import Color

class Canvas(object):
    """Canvas API helper class"""

    def __init__(self, parent, width, height, context="ctx"):
        self.obj = context
        self.code = []  # stores the code
        self.style = {}
        self.styleCache = {}  # stores the previous style applied
        self.parent = parent
        self.width = width
        self.height = height

    def write(self, text):
        self.code.append("\t" + text.replace("ctx", self.obj) + "\n")

    def output(self):
        from textwrap import dedent
        html = """
        <!DOCTYPE html>
        <html>
        <head>
            <title>Inkscape Output</title>
        </head>
        <body>
            <canvas id='canvas' width='%d' height='%d'></canvas>
            <script>
            var %s = document.getElementById("canvas").getContext("2d");
            %s
            </script>
        </body>
        </html>
        """
        return dedent(html) % (self.width, self.height, self.obj, "".join(self.code))

    def equalStyle(self, style, key):
        """Checks if the last style used is the same or there's no style yet"""
        if key in self.styleCache:
            return True
        if key not in style:
            return True
        return style[key] == self.styleCache[key]

    def beginPath(self):
        self.write("ctx.beginPath();")

    def createLinearGradient(self, href, x1, y1, x2, y2):
        data = (href, x1, y1, x2, y2)
        self.write("var %s = \
                   ctx.createLinearGradient(%f,%f,%f,%f);" % data)

    def createRadialGradient(self, href, cx1, cy1, rx, cx2, cy2, ry):
        data = (href, cx1, cy1, rx, cx2, cy2, ry)
        self.write("var %s = ctx.createRadialGradient\
                   (%f,%f,%f,%f,%f,%f);" % data)

    def addColorStop(self, href, pos, color):
        self.write("%s.addColorStop(%f, %s);" % (href, pos, color))

    def getColor(self, rgb, alpha):
        return "'{}'".format(str(Color(rgb).to_rgba(alpha)))

    def setGradient(self, href):
        """
        for stop in gstops:
            style = simplestyle.parseStyle(stop.get("style"))
            stop_color = style["stop-color"]
            opacity = style["stop-opacity"]
            color = self.getColor(stop_color, opacity)
            pos = float(stop.get("offset"))
            self.addColorStop(href, pos, color)
        """
        return None  # href

    def setOpacity(self, value):
        self.write("ctx.globalAlpha = %.1f;" % float(value))

    def setFill(self, value):
        try:
            alpha = self.style["fill-opacity"]
        except:
            alpha = 1
        if not value.startswith("url("):
            fill = self.getColor(value, alpha)
            self.write("ctx.fillStyle = %s;" % fill)

    def setStroke(self, value):
        try:
            alpha = self.style["stroke-opacity"]
        except:
            alpha = 1
        self.write("ctx.strokeStyle = %s;" % self.getColor(value, alpha))

    def setStrokeWidth(self, value):
        self.write("ctx.lineWidth = %f;" % self.parent.svg.unittouu(value))

    def setStrokeLinecap(self, value):
        self.write("ctx.lineCap = '%s';" % value)

    def setStrokeLinejoin(self, value):
        self.write("ctx.lineJoin = '%s';" % value)

    def setStrokeMiterlimit(self, value):
        self.write("ctx.miterLimit = %s;" % value)

    def setFont(self, value):
        self.write("ctx.font = \"%s\";" % value)

    def moveTo(self, x, y):
        self.write("ctx.moveTo(%f, %f);" % (x, y))

    def lineTo(self, x, y):
        self.write("ctx.lineTo(%f, %f);" % (x, y))

    def quadraticCurveTo(self, cpx, cpy, x, y):
        data = (cpx, cpy, x, y)
        self.write("ctx.quadraticCurveTo(%f, %f, %f, %f);" % data)

    def bezierCurveTo(self, x1, y1, x2, y2, x, y):
        data = (x1, y1, x2, y2, x, y)
        self.write("ctx.bezierCurveTo(%f, %f, %f, %f, %f, %f);" % data)

    def rect(self, x, y, w, h, rx=0, ry=0):
        if rx or ry:
            # rounded rectangle, starts top-left anticlockwise
            self.moveTo(x, y + ry)
            self.lineTo(x, y + h - ry)
            self.quadraticCurveTo(x, y + h, x + rx, y + h)
            self.lineTo(x + w - rx, y + h)
            self.quadraticCurveTo(x + w, y + h, x + w, y + h - ry)
            self.lineTo(x + w, y + ry)
            self.quadraticCurveTo(x + w, y, x + w - rx, y)
            self.lineTo(x + rx, y)
            self.quadraticCurveTo(x, y, x, y + ry)
        else:
            self.write("ctx.rect(%f, %f, %f, %f);" % (x, y, w, h))

    def arc(self, x, y, r, a1, a2, flag):
        data = (x, y, r, a1, a2, flag)
        self.write("ctx.arc(%f, %f, %f, %f, %.8f, %d);" % data)

    def fillText(self, text, x, y):
        self.write("ctx.fillText(\"%s\", %f, %f);" % (text, x, y))

    def translate(self, cx, cy):
        self.write("ctx.translate(%f, %f);" % (cx, cy))

    def rotate(self, angle):
        self.write("ctx.rotate(%f);" % angle)

    def scale(self, rx, ry):
        self.write("ctx.scale(%f, %f);" % (rx, ry))

    def transform(self, m11, m12, m21, m22, dx, dy):
        data = (m11, m12, m21, m22, dx, dy)
        self.write("ctx.transform(%f, %f, %f, %f, %f, %f);" % data)

    def save(self):
        self.write("ctx.save();")

    def restore(self):
        self.write("ctx.restore();")

    def closePath(self):
        if "fill" in self.style and self.style["fill"] != "none":
            self.write("ctx.fill();")
        if "stroke" in self.style and self.style["stroke"] != "none":
            self.write("ctx.stroke();")



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2006 Aaron Spike, aaron@ekips.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
import inkex
from inkex import bezier

class Flatten(inkex.EffectExtension):
    """Flattern a path"""
    def add_arguments(self, pars):
        pars.add_argument("--flatness", type=float, default=10.0, help="Minimum flattness")

    def effect(self):
        for node in self.svg.selection.filter(inkex.PathElement):
            path = node.path.to_superpath()
            bezier.cspsubdiv(path, self.options.flatness)
            newpath = []
            for subpath in path:
                first = True
                for csp in subpath:
                    cmd = 'L'
                    if first:
                        cmd = 'M'
                    first = False
                    newpath.append([cmd, [csp[1][0], csp[1][1]]])
            node.path = newpath

if __name__ == '__main__':
    Flatten().run()



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2007
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""Join paths with lines or polygons"""


import inkex

class Extrude(inkex.EffectExtension):
    def add_arguments(self, pars):
        pars.add_argument("--mode", default="Lines", help="Join paths with lines or polygons")

    def effect(self):
        paths = []
        for node in self.svg.selection.filter(inkex.PathElement):
            paths.append(node)
        if len(paths) < 2:
            raise inkex.AbortExtension("Need at least 2 paths selected")

        for path in paths:
            path.apply_transform()

        pts = [node.path.to_superpath() for node in paths]

        for n1 in range(0, len(paths)):
            for n2 in range(n1 + 1, len(paths)):
                verts = []
                for i in range(0, min(map(len, pts))):
                    comp = []
                    for j in range(0, min(len(pts[n1][i]), len(pts[n2][i]))):
                        comp.append([pts[n1][i][j][1][-2:], pts[n2][i][j][1][-2:]])
                    verts.append(comp)

                if self.options.mode.lower() == 'lines':
                    line = []
                    for comp in verts:
                        for n, v in enumerate(comp):
                            line += [('M', v[0])]
                            line += [('L', v[1])]
                    ele = inkex.PathElement()
                    paths[0].xpath('..')[0].append(ele)
                    ele.set('d', str(inkex.Path(line)))
                    style = {
                        'fill': 'none',
                        'stroke': '#000000',
                        'stroke-opacity': 1,
                        'stroke-width': self.svg.unittouu('1px'),
                    }
                    ele.set('style', str(inkex.Style(style)))
                elif self.options.mode.lower() == 'polygons':
                    g = inkex.Group()
                    style = {
                        'fill': '#000000',
                        'fill-opacity': 0.3,
                        'stroke': '#000000',
                        'stroke-opacity': 0.6,
                        'stroke-width': self.svg.unittouu('2px'),
                    }
                    g.set('style', str(inkex.Style(style)))
                    paths[0].xpath('..')[0].append(g)
                    for comp in verts:
                        for n, v in enumerate(comp):
                            nn = n + 1
                            if nn == len(comp):
                                nn = 0
                            line = []
                            line += [('M', comp[n][0])]
                            line += [('L', comp[n][1])]
                            line += [('L', comp[nn][1])]
                            line += [('L', comp[nn][0])]
                            line += [('L', comp[n][0])]
                            ele = inkex.PathElement()
                            g.append(ele)
                            ele.set('d', str(inkex.Path(line)))


if __name__ == '__main__':
    Extrude().run()



#!/usr/bin/env python
"""Reduce lightness"""

import inkex

class LessLight(inkex.ColorExtension):
    """Reduce the light of the color"""
    def modify_color(self, name, color):
        color.lightness -= int(0.05 * 255)
        return color

if __name__ == '__main__':
    LessLight().run()



#!/usr/bin/env python
"""Flip upper to lower and lower to upper cases"""

import inkex

class FlipCase(inkex.TextExtension):
    """Change the case, cHANGE THE CASE"""
    @staticmethod
    def map_char(char):
        return char.upper() if char.islower() else char.lower()

if __name__ == '__main__':
    FlipCase().run()



# coding=utf-8
#
# Copyright (C) 2005 Aaron Spike, aaron@ekips.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

import math

from bisect import bisect_left
from .utils import X, Y
from .units import convert_unit, parse_unit, render_unit

from typing import Union, Tuple, List, TypeVar, Callable, overload
hasTypes = True
Value = TypeVar('Value')
Number = TypeVar('Number', int, float)


def interpcoord(
    coord_a, # type: Number 
    coord_b, # type: Number 
    time # type: float
):
    # type: (...) -> float 
    """Interpolate single coordinate by the amount of time"""
    return coord_a + ((coord_b - coord_a) * time)


def interp(
    positions, # type: List[float]
    values, # type: List[Value]
    newpositions, # type: List[float]
    func # type: (Callable[[Value, Value, float], Value])
):
    # type: (...) -> List[Value]
    """Interpolate list with arbitrary interpolation function."""
    newvalues = []
    positions = list(map(float, positions))
    newpositions = list(map(float, newpositions))
    for pos in newpositions:
        idxl = max(0, bisect_left(positions, pos) - 1)
        idxr = min(len(positions)-1, idxl + 1)
        fraction = (pos - positions[idxl]) / (positions[idxr] - positions[idxl])
        vall = values[idxl]
        valr = values[idxr]
        newval = func(vall, valr, fraction)
        newvalues.append(newval)
    return newvalues


def interppoints(point1, point2, time):
    # type: (Tuple[float, float], Tuple[float, float], float) -> Tuple[float, float]
    """Interpolate coordinate points by amount of time"""
    return (interpcoord(point1[X], point2[X], time), interpcoord(point1[Y], point2[Y], time))


def interpunit(start, end, fraction):
    # type: (str, str, float) -> str
    """Interpolate float attributes with unit."""
    # moved here so we can call 'unittouu'
    sp, unit = parse_unit(start)
    ep = convert_unit(end, unit)
    return render_unit(interpcoord(sp, ep, fraction), unit)



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2009 Kevin Lindsey, https://github.com/thelonious/DM2SVG
#               2011 Nikita Kitaev, https://github.com/nikitakit/DM2SVG
#                    Chris Morgan,  https://gist.github.com/1471691
#               2019 Martin Owens, <doctormo@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Import a DHW file from ACECAD DigiMemo, a hardware based digitiser
"""

import struct

import inkex
from inkex import AbortExtension, errormsg, Group, Polyline

inkex.NSS['dm'] = 'http://github.com/nikitakit/DM2SVG'

class DhwInput(inkex.InputExtension):
    """Open DHW files and convert to svg on the fly"""
    template = """<svg viewBox="0 0 {w} {h}"
  fill="none" stroke="black" stroke-width="10" stroke-linecap="round" stroke-linejoin="round"
  xmlns="http://www.w3.org/2000/svg"
  xmlns:svg="http://www.w3.org/2000/svg"
  xmlns:sodipodi="http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd"
  xmlns:inkscape="http://www.inkscape.org/namespaces/inkscape"
  xmlns:dm="http://github.com/nikitakit/DM2SVG">
    <metadata>
      <dm:page
        version="{v}"
        width="{w}"
        height="{h}"
        page_type="{p}">
      </dm:page>
    </metadata>
    <rect width="{w}" height="{h}" fill="aliceblue"/>
    <g inkscape:groupmode="layer" id="layer1"></g>
</svg>"""

    def load(self, stream):
        """Load the steam as if it were an open DHW file"""
        header = list(struct.unpack('<32sBHHBxx', stream.read(40)))
        doc = header.pop(0).decode()
        if doc != 'ACECAD_DIGIMEMO_HANDWRITING_____':
            raise AbortExtension('Could not load file, not a ACECAD DHW file!')

        height = int(header[2])
        doc = self.get_template(**dict(zip(('v', 'w', 'h', 'p'), header)))
        svg = doc.getroot()

        timestamp = 0
        layer = svg.getElementById('layer1')

        while True:
            tag = stream.read(1)
            if tag == b'':
                break

            if ord(tag) <= 128:
                errormsg('Unsupported tag: {}\n'.format(tag))
                continue

            if tag == b'\x90':
                # New Layer element
                timestamp = 0
                name = 'layer{:d}'.format(ord(stream.read(1)) + 1)
                layer = svg.add(Group(inkscape_groupmode="layer", id=name))
            elif tag == b'\x88':
                # Read the timestamp next
                timestamp += ord(stream.read(1)) * 20
            else:
                # Pen down
                coords = [p for p in iter(lambda: read_point(stream, height), None)]
                # Pen up
                coords.append(read_point(stream, height))

                poly = layer.add(Polyline())
                poly.path = coords
                poly.set('dm:timestamp', timestamp)

        return doc


def read_point(stream, ymax):
    """If the next byte is a stop byte, return None. Otherwise read 4 bytes
    (in total) and return a 2D point.
    """
    # read first byte, it might be a stop byte
    x1 = struct.unpack('B', stream.read(1))[0]

    if x1 >= 0x80:
        return None

    x2, y1, y2 = struct.unpack('BBB', stream.read(3))

    x = x1 | x2 << 7
    y = y1 | y2 << 7

    return x, ymax - y

if __name__ == '__main__':
    DhwInput().run()



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2005 Aaron Spike, aaron@ekips.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

import copy
from collections import namedtuple
from itertools import combinations

import inkex
from inkex.styles import Style
from inkex.utils import pairwise
from inkex.paths import CubicSuperPath
from inkex.tween import interppoints
from inkex.bezier import csplength, cspbezsplitatlength, cspbezsplit, bezlenapprx

class Interp(inkex.EffectExtension):
    def add_arguments(self, pars):
        pars.add_argument("-e", "--exponent", type=float, default=0.0,\
            help="values other than zero give non linear interpolation")
        pars.add_argument("-s", "--steps", type=int, default=5,\
            help="number of interpolation steps")
        pars.add_argument("-m", "--method", type=int, default=2,\
            help="method of interpolation")
        pars.add_argument("-d", "--dup", type=inkex.Boolean, default=True,\
            help="duplicate endpaths")
        pars.add_argument("--style", type=inkex.Boolean, default=True,\
            help="try interpolation of some style properties")
        pars.add_argument("--zsort", type=inkex.Boolean, default=False,\
            help="use z-order instead of selection order")

    def effect(self):
        exponent = self.options.exponent
        if exponent >= 0:
            exponent += 1.0
        else:
            exponent = 1.0 / (1.0 - exponent)
        steps = [1.0 / (self.options.steps + 1.0)]
        for i in range(self.options.steps - 1):
            steps.append(steps[0] + steps[-1])
        steps = [step ** exponent for step in steps]

        if self.options.zsort:
            # work around selection order swapping with Live Preview
            objects = self.svg.get_z_selected()
        else:
            # use selection order (default)
            objects = self.svg.selected

        objects = [node for node in objects.values() if isinstance(node, inkex.PathElement)]

        # prevents modification of original objects
        objects = copy.deepcopy(objects)

        for node in objects:
            node.apply_transform()

        objectpairs = pairwise(objects, start=False) 

        for (elem1, elem2) in objectpairs:
            start = elem1.path.to_superpath()
            end = elem2.path.to_superpath()
            sst = copy.deepcopy(elem1.style)
            est = copy.deepcopy(elem2.style)
            basestyle = copy.deepcopy(sst)

            if 'stroke-width' in basestyle:
                basestyle['stroke-width'] = sst.interpolate_prop(est, 0, 'stroke-width')

            # prepare for experimental style tweening
            if self.options.style:
                styledefaults = Style(
                    {'opacity': 1.0,
                     'stroke-opacity': 1.0,
                     'fill-opacity': 1.0,
                     'stroke-width': 1.0,
                     'stroke': 'none',
                     'fill': 'none'})
                for key in styledefaults:
                    sst.setdefault(key, styledefaults[key])
                    est.setdefault(key, styledefaults[key])

                isnotplain = lambda x: not (x == 'none' or x[:1] == '#')
                isgradient = lambda x: x.startswith('url(#')

                if isgradient(sst['stroke']) and isgradient(est['stroke']):
                    strokestyle = 'gradient' 
                elif isnotplain(sst['stroke']) or isnotplain(est['stroke']) or (sst['stroke'] == 'none' and est['stroke'] == 'none'):
                    strokestyle = 'notplain' 
                else:
                    strokestyle = 'color' 

                if isgradient(sst['fill']) and isgradient(est['fill']):
                    fillstyle = 'gradient' 
                elif isnotplain(sst['fill']) or isnotplain(est['fill']) or (sst['fill'] == 'none' and est['fill'] == 'none'):
                    fillstyle = 'notplain' 
                else:
                    fillstyle = 'color'

                if strokestyle == 'color':
                    if sst['stroke'] == 'none':
                        sst['stroke-width'] = '0.0'
                        sst['stroke-opacity'] = '0.0'
                        sst['stroke'] = est['stroke']
                    elif est['stroke'] == 'none':
                        est['stroke-width'] = '0.0'
                        est['stroke-opacity'] = '0.0'
                        est['stroke'] = sst['stroke']

                if fillstyle == 'color':
                    if sst['fill'] == 'none':
                        sst['fill-opacity'] = '0.0'
                        sst['fill'] = est['fill']
                    elif est['fill'] == 'none':
                        est['fill-opacity'] = '0.0'
                        est['fill'] = sst['fill']

            if self.options.method == 2:
                # subdivide both paths into segments of relatively equal lengths
                slengths, stotal = csplength(start)
                elengths, etotal = csplength(end)
                lengths = {}
                t = 0
                for sp in slengths:
                    for l in sp:
                        t += l / stotal
                        lengths.setdefault(t, 0)
                        lengths[t] += 1
                t = 0
                for sp in elengths:
                    for l in sp:
                        t += l / etotal
                        lengths.setdefault(t, 0)
                        lengths[t] += -1
                sadd = [k for (k, v) in lengths.items() if v < 0]
                sadd.sort()
                eadd = [k for (k, v) in lengths.items() if v > 0]
                eadd.sort()

                t = 0
                s = [[]]
                for sp in slengths:
                    if not start[0]:
                        s.append(start.pop(0))
                    s[-1].append(start[0].pop(0))
                    for l in sp:
                        pt = t
                        t += l / stotal
                        if sadd and t > sadd[0]:
                            while sadd and sadd[0] < t:
                                nt = (sadd[0] - pt) / (t - pt)
                                bezes = cspbezsplitatlength(s[-1][-1][:], start[0][0][:], nt)
                                s[-1][-1:] = bezes[:2]
                                start[0][0] = bezes[2]
                                pt = sadd.pop(0)
                        s[-1].append(start[0].pop(0))
                t = 0
                e = [[]]
                for sp in elengths:
                    if not end[0]:
                        e.append(end.pop(0))
                    e[-1].append(end[0].pop(0))
                    for l in sp:
                        pt = t
                        t += l / etotal
                        if eadd and t > eadd[0]:
                            while eadd and eadd[0] < t:
                                nt = (eadd[0] - pt) / (t - pt)
                                bezes = cspbezsplitatlength(e[-1][-1][:], end[0][0][:], nt)
                                e[-1][-1:] = bezes[:2]
                                end[0][0] = bezes[2]
                                pt = eadd.pop(0)
                        e[-1].append(end[0].pop(0))
                start = s[:]
                end = e[:]
            else:
                # which path has fewer segments?
                lengthdiff = len(start) - len(end)
                # swap shortest first
                if lengthdiff > 0:
                    start, end = end, start
                # subdivide the shorter path
                for x in range(abs(lengthdiff)):
                    maxlen = 0
                    subpath = 0
                    segment = 0
                    for y in range(len(start)):
                        for z in range(1, len(start[y])):
                            leng = bezlenapprx(start[y][z - 1], start[y][z])
                            if leng > maxlen:
                                maxlen = leng
                                subpath = y
                                segment = z
                    sp1, sp2 = start[subpath][segment - 1:segment + 1]
                    start[subpath][segment - 1:segment + 1] = cspbezsplit(sp1, sp2)
                # if swapped, swap them back
                if lengthdiff > 0:
                    start, end = end, start

            # break paths so that corresponding subpaths have an equal number of segments
            s = [[]]
            e = [[]]
            while start and end:
                if start[0] and end[0]:
                    s[-1].append(start[0].pop(0))
                    e[-1].append(end[0].pop(0))
                elif end[0]:
                    s.append(start.pop(0))
                    e[-1].append(end[0][0])
                    e.append([end[0].pop(0)])
                elif start[0]:
                    e.append(end.pop(0))
                    s[-1].append(start[0][0])
                    s.append([start[0].pop(0)])
                else:
                    s.append(start.pop(0))
                    e.append(end.pop(0))

            if self.options.dup:
                steps = [0] + steps + [1]
            # create an interpolated path for each interval
            group = self.svg.get_current_layer().add(inkex.Group())
            for time in steps:
                interp = []
                # process subpaths
                for ssp, esp in zip(s, e):
                    if not (ssp or esp):
                        break
                    interp.append([])
                    # process superpoints
                    for sp, ep in zip(ssp, esp):
                        if not (sp or ep):
                            break
                        interp[-1].append([])
                        # process points
                        for p1, p2 in zip(sp, ep):
                            if not (sp or ep):
                                break
                            interp[-1][-1].append(interppoints(p1, p2, time))

                # remove final subpath if empty.
                if not interp[-1]:
                    del interp[-1]

                # basic style interpolation
                if self.options.style:
                    basestyle.update(sst.interpolate(est, time))
                    for prop in ['stroke', 'fill']:
                        if isgradient(sst[prop]) and isgradient(est[prop]):
                            gradid1 = sst[prop][4:-1]
                            gradid2 = est[prop][4:-1]
                            grad1 = self.svg.getElementById(gradid1)
                            grad2 = self.svg.getElementById(gradid2)
                            newgrad = grad1.interpolate(grad2, time)
                            stops, orientation = newgrad.stops_and_orientation()
                            self.svg.defs.add(orientation)
                            basestyle[prop] = orientation.get_id(as_url=2)
                            if len(stops):
                                self.svg.defs.add(stops, orientation)
                                orientation.set('xlink:href', stops.get_id(as_url=1))

                new = group.add(inkex.PathElement())
                new.style = basestyle
                new.path = CubicSuperPath(interp)


if __name__ == '__main__':
    Interp().run()



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2016 Richard White, rwhite8282@gmail.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
An Inkscape extension that creates a frame around a selected object.
"""

import inkex
from inkex import Group, PathElement, ClipPath

def size_box(box, delta):
    """ Returns a box with an altered size.
    delta -- The amount the box should grow.
    Returns a box with an altered size.
    """
    return (box.x.minimum - delta, box.x.maximum + delta,
            box.y.minimum - delta, box.y.maximum + delta)


# Frame maker Inkscape effect extension
class Frame(inkex.EffectExtension):
    """
    An Inkscape extension that creates a frame around a selected object.
    """
    def add_arguments(self, pars):
        # Parse the options.
        pars.add_argument('--tab', default='object')
        pars.add_argument('--clip', type=inkex.Boolean, default=False)
        pars.add_argument('--corner_radius', type=int, default=0)
        pars.add_argument('--fill_color', type=inkex.Color, default=inkex.Color(0))
        pars.add_argument('--group', type=inkex.Boolean, default=False)
        pars.add_argument('--position', default='outside')
        pars.add_argument('--stroke_color', type=inkex.Color, default=inkex.Color(0))
        pars.add_argument('--width', type=float, default=2.0)

    def add_clip(self, node, clip_path):
        """ Adds a new clip path node to the defs and sets
                the clip-path on the node.
            node -- The node that will be clipped.
            clip_path -- The clip path object.
        """
        clip = ClipPath()
        clip.append(PathElement(d=str(clip_path.path)))
        clip_id = self.svg.get_unique_id('clipPath')
        clip.set('id', clip_id)
        self.svg.defs.append(clip)
        node.set('clip-path', 'url(#{})'.format(str(clip_id)))

    def add_frame(self, name, box, style, radius=0):
        """
            name -- The name of the new frame object.
            box -- The boundary box of the node.
            style -- The style used to draw the path.
            radius -- The corner radius of the frame.
            returns a new frame node.
        """
        r = min([radius, (abs(box[1] - box[0]) / 2), (abs(box[3] - box[2]) / 2)])
        if radius > 0:
            d = ' '.join(str(x) for x in
                         ['M', box[0], (box[2] + r),
                          'A', r, r, '0 0 1', (box[0] + r), box[2],
                          'L', (box[1] - r), box[2],
                          'A', r, r, '0 0 1', box[1], (box[2] + r),
                          'L', box[1], (box[3] - r),
                          'A', r, r, '0 0 1', (box[1] - r), box[3],
                          'L', (box[0] + r), box[3],
                          'A', r, r, '0 0 1', box[0], (box[3] - r),
                          'Z'])
        else:
            d = ' '.join(str(x) for x in
                         ['M', box[0], box[2],
                          'L', box[1], box[2],
                          'L', box[1], box[3],
                          'L', box[0], box[3],
                          'Z'])

        elem = PathElement()
        elem.style = style
        elem.label = name
        elem.path = d
        return elem

    def effect(self):
        """Performs the effect."""
        # Determine common properties.
        width = self.options.width
        style = inkex.Style({'stroke-width': width})
        style.set_color(self.options.fill_color, 'fill')
        style.set_color(self.options.stroke_color, 'stroke')
        layer = self.svg.get_current_layer()

        for node in self.svg.selected.values():
            box = node.bounding_box()
            if self.options.position == 'outside':
                box = size_box(box, (width / 2))
            else:
                box = size_box(box, -(width / 2))

            frame = self.add_frame("Frame", box, style, self.options.corner_radius)
            if self.options.clip:
                self.add_clip(node, frame)
            if self.options.group:
                group = layer.add(Group())
                group.append(node)
                group.append(frame)
            else:
                layer.append(frame)
        return None

if __name__ == '__main__':
    Frame().run()



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2005 Pim Snel, pim@lingewoud.com
# Copyright (C) 2008 Aaron Spike, aaron@ekips.org
# Copyright (C) 2011 Nicolas Dufour, nicoduf@yahoo.fr
#
#    * Fix for a bug related to special characters in the path (LP #456248).
#    * Fix for Windows support (LP #391307 ).
#    * Font list and image directory features.
#
# this is  the first Python script  ever created
# its based on embedimage.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# TODOs
# - fix bug: not saving existing .zip after a Collect for Output is run
#     this bug occurs because after running an effect extension the inkscape:output_extension is reset to svg.inkscape
#     the file name is still xxx.zip. after saving again the file xxx.zip is written with a plain .svg which
#     looks like a corrupt zip
# - maybe add better extension
# - consider switching to lzma in order to allow cross platform compression with no encoding problem...
#
"""
An extension which collects all images to the documents directory and
creates a zip archive containing all images and the document
"""

import os
import shutil
import tempfile
import zipfile

import inkex
from inkex import TextElement, Tspan, FlowRoot, FlowPara, FlowSpan

try:  # PY2
    from urllib import url2pathname
    from urlparse import urlparse
except ImportError:  # PY3
    from urllib.parse import urlparse
    from urllib.request import url2pathname

ENCODING = "cp437" if os.name == 'nt' else "latin-1"

class CompressedMedia(inkex.OutputExtension):
    """Output a compressed file"""
    def add_arguments(self, pars):
        pars.add_argument("--image_dir", help="Image directory")
        pars.add_argument("--font_list", type=inkex.Boolean, help="Add font list")

    def collect_images(self, docname, z):
        """
        Collects all images in the document
        and copy them to the temporary directory.
        """
        imgdir = self.options.image_dir

        for node in self.svg.xpath('//svg:image'):
            xlink = node.get('xlink:href')
            if xlink[:4] != 'data':
                absref = node.get('sodipodi:absref')
                url = urlparse(xlink)
                href = url2pathname(url.path)

                if href is not None and os.path.isfile(href):
                    absref = os.path.realpath(href)

                image_path = os.path.join(imgdir, os.path.basename(absref))

                if os.path.isfile(absref):
                    shutil.copy(absref, self.tmp_dir)
                    z.write(absref, image_path.encode(ENCODING))
                elif os.path.isfile(os.path.join(self.tmp_dir, absref)):
                    # TODO: please explain why this clause is necessary
                    shutil.copy(os.path.join(self.tmp_dir, absref), self.tmp_dir)
                    z.write(os.path.join(self.tmp_dir, absref), image_path.encode(ENCODING))
                else:
                    inkex.errormsg('Could not locate file: %s' % absref)

                node.set('xlink:href', image_path)

    def collect_svg(self, docstripped, z):
        """
        Copy SVG document to the temporary directory
        and add it to the temporary compressed file
        """
        dst_file = os.path.join(self.tmp_dir, docstripped)
        with open(dst_file, 'wb') as stream:
            self.document.write(stream)
        z.write(dst_file, docstripped + '.svg')

    def is_text(self, node):
        """
        Returns true if the tag in question is an element that
        can hold text.
        """
        return isinstance(node, (TextElement, Tspan, FlowRoot, FlowPara, FlowSpan))

    def get_fonts(self, node):
        """
        Given a node, returns a list containing all the fonts that
        the node is using.
        """
        fonts = []
        s = ''
        if 'style' in node.attrib:
            s = dict(inkex.Style.parse_str(node.attrib['style']))
        if not s:
            return fonts

        if 'font-family' in s:
            if 'font-weight' in s:
                fonts.append(s['font-family'] + ' ' + s['font-weight'])
            else:
                fonts.append(s['font-family'])
        elif '-inkscape-font-specification' in s:
            fonts.append(s['-inkscape-font-specification'])
        return fonts

    def list_fonts(self, z):
        """
        Walks through nodes, building a list of all fonts found, then
        reports to the user with that list.
        Based on Craig Marshall's replace_font.py
        """
        nodes = []
        items = self.document.getroot().getiterator()
        nodes.extend(filter(self.is_text, items))
        fonts_found = []
        for node in nodes:
            for f in self.get_fonts(node):
                if not f in fonts_found:
                    fonts_found.append(f)
        findings = sorted(fonts_found)
        # Write list to the temporary compressed file
        filename = 'fontlist.txt'
        dst_file = os.path.join(self.tmp_dir, filename)
        with open(dst_file, 'w') as stream:
            if len(findings) == 0:
                stream.write("Didn't find any fonts in this document/selection.")
            else:
                if len(findings) == 1:
                    stream.write("Found the following font only: %s" % findings[0])
                else:
                    stream.write("Found the following fonts:\n%s" % '\n'.join(findings))
        z.write(dst_file, filename)

    def save(self, stream):
        docname = self.svg.get('sodipodi:docname')

        if docname is None:
            docname = self.options.input_file

        # TODO: replace whatever extension
        docstripped = os.path.basename(docname.replace('.zip', ''))
        docstripped = docstripped.replace('.svg', '')
        docstripped = docstripped.replace('.svgz', '')

        # Create os temp dir
        self.tmp_dir = tempfile.mkdtemp()

        # Create destination zip in same directory as the document
        with zipfile.ZipFile(stream, 'w') as z:
            self.collect_images(docname, z)
            self.collect_svg(docstripped, z)
            if self.options.font_list:
                self.list_fonts(z)

if __name__ == '__main__':
    CompressedMedia().run()



# coding=utf-8
#
# Copyright (C) 2010 Nick Drobchenko, nick@cnc-club.ru
# Copyright (C) 2005 Aaron Spike, aaron@ekips.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Allow extensions to translate messages.
"""

import gettext
import os

# Get gettext domain and matching locale directory for translation of extensions strings
# (both environment variables are set by Inkscape)
GETTEXT_DOMAIN = os.environ.get('INKEX_GETTEXT_DOMAIN')
GETTEXT_DIRECTORY = os.environ.get('INKEX_GETTEXT_DIRECTORY')

# INKSCAPE_LOCALEDIR can be used to override the default locale directory Inkscape uses
INKSCAPE_LOCALEDIR = os.environ.get('INKSCAPE_LOCALEDIR')

def localize(domain=GETTEXT_DOMAIN, localedir=GETTEXT_DIRECTORY):
    """Configure gettext and install _() function into builtins namespace for easy access"""

    # Do not enable translation if GETTEXT_DOMAIN is unset.
    # This is the case when translationdomain="none", but also when no catalog was found.
    # Install a NullTranslation just to be sure (so we do not get errors about undefined '_')
    if domain is None:
        gettext.NullTranslations().install()
        return

    # Use the default system locale by default,
    # but prefer LANGUAGE environment variable (which is set by Inkscape according to UI language)
    languages = None

    trans = gettext.translation(domain, localedir, languages, fallback=True)
    trans.install()



def inkex_localize():
    """
    Return internal Translations instance for translation of the inkex module itself
    Those will always use the 'inkscape' domain and attempt to lookup the same catalog Inkscape uses
    """

    domain = 'inkscape'
    localedir = INKSCAPE_LOCALEDIR
    languages = None

    return gettext.translation(domain, localedir, languages, fallback=True)

inkex_gettext = inkex_localize().gettext  # pylint: disable=invalid-name



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2009 Aurelio A. Heckert, aurium (a) gmail dot com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

import os
import re
import sys

import inkex
from inkex import Script

class InkWebEffect(inkex.EffectExtension):
    reUpdateJS = '/\\*\\s* inkweb.js [^*]* InkWebEffect:AutoUpdate \\s*\\*/'
    def effect(self):
        pass

    def mustAddInkWebJSCode(self, script):
        if not script.text:
            return True
        if re.search(self.reUpdateJS, script.text):
            return True
        return False

    def addInkWebJSCode(self, script):
        with open(os.path.join(sys.path[0], "inkweb.js")) as js:
            script.set_text("\n/* inkweb.js - InkWebEffect:AutoUpdate */\n" + js.read())

    def ensureInkWebSupport(self):
        # Search for the script tag with the inkweb.js code:
        script = None
        for child in self.svg.xpath('//svg:script'):
            if re.search(self.reUpdateJS, child.text):
                script = child

        if script is None:
            script = Script(id="inkwebjs", type="text/javascript")
            self.svg.insert(0, script)

        if self.mustAddInkWebJSCode(script):
            self.addInkWebJSCode(script)
